import { ConfigApi } from "./ConfigApi";
import { IConfig } from "../doc/IConfig";
import { IConfigVault } from "../doc/IConfigVault";
import { IConfigProfile } from "../doc/IConfigProfile";
/**
 * API Class for manipulating config layers.
 */
export declare class ConfigSecure extends ConfigApi {
    private mLoadFailed;
    /**
     * Load the secure application properties from secure storage using the
     * specified vault interface. The vault interface is placed into our
     * Config object. The secure values are placed into our Config layers.
     *
     * @param vault Interface for loading and saving to secure storage.
     */
    load(vault?: IConfigVault): Promise<void>;
    /**
     * Copy secure config properties from cached secure properties into the
     * specified layer. To load secure config properties directly from the
     * vault, use the asynchronous method `load` instead.
     *
     * @internal
     * @param opts The user and global flags that specify one of the four
     *             config files (aka layers).
     */
    loadCached(opts?: {
        user: boolean;
        global: boolean;
    }): void;
    /**
     * Save the secure application properties into secure storage using
     * the vault interface from our config object.
     *
     * @param allLayers Specify true to save all config layers instead of only the active one
     */
    save(allLayers?: boolean): Promise<void>;
    /**
     * Save secure properties to the vault without rebuilding the JSON object.
     * @internal
     */
    directSave(): Promise<void>;
    /**
     * Copy secure config properties from the specified layer into cached
     * secure properties. To save secure config properties directly to the
     * vault, use the asynchronous method `save` instead.
     *
     * Warning: Do not pass an `IConfigLayer` object into this method unless
     * you want its properties to be edited.
     *
     * @internal
     * @param opts The user and global flags that specify one of the four
     *             config files (aka layers).
     * @param opts.properties `IConfig` object cloned from the specified layer.
     *                        If specified, secure properties will be removed.
     */
    cacheAndPrune(opts?: {
        user: boolean;
        global: boolean;
        properties?: IConfig;
    }): void;
    /**
     * List full paths of all secure properties found in a team config file.
     *
     * @param opts The user and global flags that specify one of the four
     *             config files (aka layers).
     * @returns Array of secure property paths
     *          (e.g., "profiles.lpar1.properties.password")
     */
    secureFields(opts?: {
        user: boolean;
        global: boolean;
    }): string[];
    /**
     * List names of secure properties for a profile. They may be defined at
     * the profile's level, or at a higher level if the config is nested.
     * @param profileName Profile name to search for
     * @returns Array of secure property names
     */
    securePropsForProfile(profileName: string): string[];
    /**
     * Recursively find secure property paths inside a team config
     * "profiles" object.
     * @internal
     * @param profiles The "profiles" object that is present at the top level
     *                 of team config files, and may also be present at lower
     *                 levels.
     * @param path The JSON path to the "profiles" object
     * @returns Array of secure property paths
     */
    findSecure(profiles: {
        [key: string]: IConfigProfile;
    }, path: string): string[];
    /**
     * Retrieve info that can be used to store a profile property securely.
     *
     * For example, to securely store "profiles.lpar1.properties.password", the
     * name "password" would be stored in "profiles.lpar1.secure".
     *
     * @internal
     * @param propertyPath The full path of the profile property
     * @param findUp Specify true to search up in the config file for higher level secure arrays
     * @returns Object with the following attributes:
     *  - `path` The JSON path of the secure array
     *  - `prop` The name of the property
     */
    secureInfoForProp(propertyPath: string, findUp?: boolean): {
        path: string;
        prop: string;
    };
    /**
     * Delete secure properties stored for team config files that do not exist.
     * @returns Array of file paths for which properties were deleted
     */
    rmUnusedProps(): string[];
    /**
     * Return true if the secure load method was called and threw an error, or
     * it was never called because the CredentialManager failed to initialize.
     */
    get loadFailed(): boolean;
}
