"use strict";
/*
* This program and the accompanying materials are made available under the terms of the
* Eclipse Public License v2.0 which accompanies this distribution, and is available at
* https://www.eclipse.org/legal/epl-v20.html
*
* SPDX-License-Identifier: EPL-2.0
*
* Copyright Contributors to the Zowe Project.
*
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoggerUtils = void 0;
const EnvironmentalVariableSettings_1 = require("../../imperative/src/env/EnvironmentalVariableSettings");
const CliUtils_1 = require("../../utilities/src/CliUtils");
const ImperativeConfig_1 = require("../../utilities/src/ImperativeConfig");
const lodash = require("lodash");
class LoggerUtils {
    /**
     * Copy and censor any sensitive CLI arguments before logging/printing
     * @param {string[]} args
     * @returns {string[]}
     */
    static censorCLIArgs(args) {
        const newArgs = JSON.parse(JSON.stringify(args));
        const censoredValues = LoggerUtils.CENSORED_OPTIONS.map(CliUtils_1.CliUtils.getDashFormOfOption);
        for (const value of censoredValues) {
            if (args.indexOf(value) >= 0) {
                const valueIndex = args.indexOf(value);
                if (valueIndex < args.length - 1) {
                    newArgs[valueIndex + 1] = LoggerUtils.CENSOR_RESPONSE; // censor the argument after the option name
                }
            }
        }
        return newArgs;
    }
    /**
     * Copy and censor a yargs argument object before logging
     * @param {yargs.Arguments} args the args to censor
     * @returns {yargs.Arguments}  a censored copy of the arguments
     */
    static censorYargsArguments(args) {
        const newArgs = JSON.parse(JSON.stringify(args));
        for (const optionName of Object.keys(newArgs)) {
            if (LoggerUtils.CENSORED_OPTIONS.indexOf(optionName) >= 0) {
                const valueToCensor = newArgs[optionName];
                newArgs[optionName] = LoggerUtils.CENSOR_RESPONSE;
                for (const checkAliasKey of Object.keys(newArgs)) {
                    if (newArgs[checkAliasKey] === valueToCensor) {
                        newArgs[checkAliasKey] = LoggerUtils.CENSOR_RESPONSE;
                    }
                }
            }
        }
        return newArgs;
    }
    static get config() {
        if (LoggerUtils.mConfig == null)
            LoggerUtils.mConfig = ImperativeConfig_1.ImperativeConfig.instance.config;
        return LoggerUtils.mConfig;
    }
    static get layer() {
        // if (LoggerUtils.mLayer == null) LoggerUtils.mLayer = LoggerUtils.config.api.layers.get();
        // Have to get a new copy every time because of how secure properties get lazy-loaded
        LoggerUtils.mLayer = LoggerUtils.config.api.layers.get();
        return LoggerUtils.mLayer;
    }
    static get secureFields() {
        if (LoggerUtils.mSecureFields == null)
            LoggerUtils.mSecureFields = LoggerUtils.config.api.secure.secureFields();
        return LoggerUtils.mSecureFields;
    }
    static get profileSchemas() {
        var _a, _b;
        if (LoggerUtils.mProfiles == null)
            LoggerUtils.mProfiles = (_b = (_a = ImperativeConfig_1.ImperativeConfig.instance.loadedConfig) === null || _a === void 0 ? void 0 : _a.profiles) !== null && _b !== void 0 ? _b : [];
        return LoggerUtils.mProfiles;
    }
    static setProfileSchemas(_schemas) {
        if (LoggerUtils.mProfiles == null) {
            LoggerUtils.mProfiles = [];
        }
        _schemas.forEach((v) => {
            LoggerUtils.mProfiles.push({ type: v.type, schema: v });
        });
    }
    /**
     * Copy and censor any sensitive CLI arguments before logging/printing
     * @param {string} data
     * @returns {string}
     */
    static censorRawData(data, category = "") {
        var _a;
        // Return the data if not using config
        if (!((_a = ImperativeConfig_1.ImperativeConfig.instance.config) === null || _a === void 0 ? void 0 : _a.exists))
            return data;
        // Return the data if we are printing to the console and masking is disabled
        const envMaskOutput = EnvironmentalVariableSettings_1.EnvironmentalVariableSettings.read(ImperativeConfig_1.ImperativeConfig.instance.envVariablePrefix).maskOutput.value;
        // Hardcoding "console" instead of using Logger.DEFAULT_CONSOLE_NAME because of circular dependencies
        if ((category === "console" || category === "json") && envMaskOutput.toUpperCase() === "FALSE")
            return data;
        let newData = data;
        const layer = LoggerUtils.layer;
        for (const prop of LoggerUtils.secureFields) {
            const sec = lodash.get(layer.properties, prop);
            if (sec && typeof sec !== "object" && !LoggerUtils.isSpecialValue(prop))
                newData = newData.replace(new RegExp(sec, "gi"), LoggerUtils.CENSOR_RESPONSE);
        }
        return newData;
    }
}
exports.LoggerUtils = LoggerUtils;
LoggerUtils.CENSOR_RESPONSE = "****";
/**
 * NOTE(Kelosky): Ideally we might have a consolidated list for secure fields, but for now we'll just
 * make sure they're collocated within the same class.
 */
LoggerUtils.CENSORED_OPTIONS = ["auth", "p", "pass", "password", "passphrase", "credentials",
    "authentication", "basic-auth", "basicAuth", "tv", "token-value", "tokenValue",
    "cert-file-passphrase", "certFilePassphrase"];
LoggerUtils.SECURE_PROMPT_OPTIONS = ["user", "password", "tokenValue", "passphrase"];
/**
 * Singleton implementation of an internal reference of ImperativeConfig.instance.config
 */
LoggerUtils.mConfig = null;
/**
 * Singleton implementation of an internal reference to the active layer
 * This should help with performance since one a single copy will be created for censoring data
 */
LoggerUtils.mLayer = null;
/**
 * Singleton implementation of an internal reference to the secure fields stored in the config
 */
LoggerUtils.mSecureFields = null;
/**
 * Singleton implementation of an internal reference to the loaded profiles
 */
LoggerUtils.mProfiles = null;
/**
 * Specifies whether a given property path (e.g. "profiles.lpar1.properties.host") is a special value or not.
 * Special value: Refers to any value defined as secure in the schema definition.
 *                These values should be already masked by the application (and/or plugin) developer.
 * @param prop Property path to determine if it is a special value
 * @returns True - if the given property is to be treated as a special value; False - otherwise
 */
LoggerUtils.isSpecialValue = (prop) => {
    let specialValues = ["user", "password", "tokenValue", "keyPassphrase"];
    /**
     * Helper function that return a list of all optionDefinition names for a given ICommandProfileProperty
     * @param prop profile property to get the optionDefinition names from
     * @returns List of optionDefinition names
     */
    const getPropertyNames = (prop) => {
        var _a, _b;
        const ret = [];
        ret.push((_a = prop.optionDefinition) === null || _a === void 0 ? void 0 : _a.name);
        (_b = prop.optionDefinitions) === null || _b === void 0 ? void 0 : _b.map(opDef => ret.push(opDef.name));
        return ret;
    };
    for (const profile of LoggerUtils.profileSchemas) {
        // eslint-disable-next-line unused-imports/no-unused-vars
        for (const [_, prop] of Object.entries(profile.schema.properties)) {
            if (prop.secure)
                specialValues = lodash.union(specialValues, getPropertyNames(prop));
        }
    }
    // TODO: How to handle DNS resolution (using a wrong port)
    //  ex: zowe jobs list jobs --port 12345
    //      May print the IP address of the given host if the resolved IP:port combination is not valid
    for (const v of specialValues) {
        if (prop.endsWith(`.${v}`))
            return true;
    }
    return false;
};
//# sourceMappingURL=LoggerUtils.js.map